(* $Id: godi_query.mli 964 2012-05-25 23:41:21Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

(* Query the databases of available and installed packages *)

open Godi_types
open Godi_exec
open Godi_versions

val godi_conf : unit -> string
  (* Returns the location of godi.conf *)

val profile_conf : unit -> string
  (* Returns the location of profile.conf *)

val localbase : unit -> string
  (* Returns the value of LOCALBASE *)

val localbase_filename : unit -> Godi_file.filename
  (* same as filename *)

val pkgdbdir : unit -> string
  (* Returns the value of PKG_DBDIR *)

val pkgdbdir_filename : unit -> Godi_file.filename
  (* same as filename *)

val buildbase : unit -> string
  (* Returns the build directory, i.e. $LOCALBASE/build *)

val buildbase_filename : unit -> Godi_file.filename
  (* Same as filename *)

val shlib_type : unit -> string
  (* Returns the type of shared library *)

val profile_name : unit -> string
  (* The name of the current profile *)

val profile_filename : unit -> Godi_file.filename
  (* The path of the profile directory. Append e.g. "profile.ini" *)

val wget : unit -> string
  (* The absolute path to the wget program *)

val wget_options : unit -> string
  (* Options for calling wget *)

val user : unit -> string option
  (* returns the user name to access files (GODI_USER)  *)

val store_passwords : unit -> bool
  (* whether to store web passwords (GODI_STORE_PASSWORDS) *)

val godi_binpkg_path : unit -> string list
  (* returns the URLs in GODI_BINPKG (or [] if not set) *)


val have_ocamlopt : unit -> bool
val have_ocamloptp : unit -> bool
val have_ocamldll : unit -> bool
  (* Returns whether the corresponding features are available *)

val get_tools_makefile : unit -> string
  (* Returns a path to the godi_tools.mk file *)

(* Note: if one of the above functions runs into a problem, User_error
 * will be raised
 *)

val read_text_file : ?pref_user_error:bool -> ?default:string -> 
                     Godi_file.filename -> string
  (* Reads the text file, and returns its contents as string. 
   * If the file does not exist, and a [default] is passed, that is
   * returned instead. Otherwise, it is an error if the file does not exist.
   *)


val read_optional_text_file : ?pref_user_error:bool -> 
                                Godi_file.filename -> string option
  (* Reads the file, and returns its contents as string. If the file
   * does not exist, [None] is returned.
   *)


val query_makefile :
      ?pref_user_error:bool ->
      ?godi_conf:string ->
      ?makefile:string ->
      ?uncached:bool ->
      console ->
      Godi_file.filename  ->
      string list ->
        string list
  (* query_makefile directory varnames: chdir to directory, and query the
   * values of varnames using "show-vars".
   * TODO: Move to Godi_make.
   *
   * pref_user_error: Whether to prefer User_error over other exceptions.
   *
   * uncached: the parsed files are not entered into the shared cache
   *)


val scan_source_packages :
      ?rescan:bool ->
      console ->
	package list

val forget_source_packages : unit -> unit

val get_installed_packages :
      ?verbose:bool ->
      console ->
	package list
  (** Returns the in-memory representation of the installed packages.
   * Note that there is always only one global list of installed packages,
   * because there is only one package database.
   *
   * @param verbose If true, messages are printed to the protocol when
   * the list is reloaded or reconciled.
   *)

val get_this_installed_package : ?verbose:bool ->
				 console -> string -> package
  (** Returns the package with the passed name (w/o version suffix).
   * Raises [Not_found] if the package does not exist.
   *
   * @param verbose See [get_installed_packages]
   *)


val notify_about_change_of_installed_packages :
      console -> [ `Reload | `Reconcile ] -> unit
  (** Notifies that the packages have been added or deleted, and that the
   * in-memory representation of the installation set is out of date.
   *
   * - [`Reload]: The next call of [get_installed_packages] will reload
   *   all package definitions
   * - [`Reconcile]: The next call of [get_installed_packages] will
   *   check which packages are new, deleted, or modified, by comparing
   *   the files on disk with the in-memory representation, and will only
   *   load the changes.
   *
   * [`Reload] beats [`Reconcile].
   *)

val get_binary_package : Godi_file.filename -> package
  (** Reads the metainformation at the beginning of a .tgz file *)

val get_remote_binary_package : 
       Godi_plist.inst_plist_element list -> string -> package
  (** [get_remote_binary_package inst_plist url]: Makes a binary package
      object from:
       - inst_plist: The PLIST directives read from PKGMANIFEST
       - url: The URL of the package

      Certain properties are first available when the package has been
      downloaded (require_binary_archive):
       - install_message
       - build_version
       - build_info
       - requirements_script
       - install_script
       - deinstall_script
   *)

val binary_version_digest : package -> string
  (** Returns the hex code that is included in file names of binary pkgs *)

val binary_file_base : package -> string
  (** The base filename for binary packages (w/ "@BUILD:" but w/o ".tgz")
   *)

val scan_binary_packages : console -> string -> package list
  (** Scans the binary packages at this URL *)

val scan_all_binary_packages : console -> package list
  (** Scans the binary packages at all locations in the env var
      GODI_BINPKG
   *)


val matches_dep : package -> dependency -> bool
  (** [matches_dep pkg dep]: whether this package fulfills the dependency
   *)

(*
val error_msg : exn -> string
*)
